Index: linux-4.19.118/net/ipv4/ip_fragment.c
===================================================================
--- linux-4.19.118.orig/net/ipv4/ip_fragment.c
+++ linux-4.19.118/net/ipv4/ip_fragment.c
@@ -55,6 +55,16 @@
  * code now. If you change something here, _PLEASE_ update ipv6/reassembly.c
  * as well. Or notify me, at least. --ANK
  */
+
+/*
+ * used in ip_output.c
+ *
+ * 1-7 turn on auto frag sizing
+ * >7 all fragments will be at most this size
+ */
+int sysctl_ipfrag_size  __read_mostly = 0;
+EXPORT_SYMBOL(sysctl_ipfrag_size);
+
 static const char ip_frag_cache_name[] = "ip4-frags";
 
 /* Describe an entry in the "incomplete datagrams" queue. */
@@ -566,7 +576,7 @@ static struct ctl_table ip4_frags_ns_ctl
 		.data		= &init_net.ipv4.frags.timeout,
 		.maxlen		= sizeof(int),
 		.mode		= 0644,
-		.proc_handler	= proc_dointvec_jiffies,
+		.proc_handler	= proc_dointvec_jiffies
 	},
 	{
 		.procname	= "ipfrag_max_dist",
@@ -574,8 +584,15 @@ static struct ctl_table ip4_frags_ns_ctl
 		.maxlen		= sizeof(int),
 		.mode		= 0644,
 		.proc_handler	= proc_dointvec_minmax,
-		.extra1		= &dist_min,
+		.extra1		= &dist_min
 	},
+	{
+		.procname	= "ipfrag_size",
+		.data		= &sysctl_ipfrag_size,
+		.maxlen		= sizeof(int),
+		.mode		= 0644,
+		.proc_handler	= proc_dointvec_minmax
+ 	},
 	{ }
 };
 
Index: linux-4.19.118/net/ipv4/ip_output.c
===================================================================
--- linux-4.19.118.orig/net/ipv4/ip_output.c
+++ linux-4.19.118/net/ipv4/ip_output.c
@@ -81,6 +81,9 @@
 #include <linux/netlink.h>
 #include <linux/tcp.h>
 
+/* only thing we need from ip_fragment.c */
+extern int sysctl_ipfrag_size;
+
 static int
 ip_fragment(struct net *net, struct sock *sk, struct sk_buff *skb,
 	    unsigned int mtu,
@@ -575,7 +578,7 @@ int ip_do_fragment(struct net *net, stru
 	struct iphdr *iph;
 	int ptr;
 	struct sk_buff *skb2;
-	unsigned int mtu, hlen, left, len, ll_rs;
+	unsigned int mtu, hlen, left, len, flen, ll_rs;
 	int offset;
 	__be16 not_last_frag;
 	struct rtable *rt = skb_rtable(skb);
@@ -623,6 +626,9 @@ int ip_do_fragment(struct net *net, stru
 		    skb_headroom(skb) < ll_rs)
 			goto slow_path;
 
+		if (sysctl_ipfrag_size > 7 && (first_len - hlen > sysctl_ipfrag_size))
+			goto slow_path;
+
 		skb_walk_frags(skb, frag) {
 			/* Correct geometry. */
 			if (frag->len > mtu ||
@@ -630,6 +636,13 @@ int ip_do_fragment(struct net *net, stru
 			    skb_headroom(frag) < hlen + ll_rs)
 				goto slow_path_clean;
 
+			if (sysctl_ipfrag_size) {
+				if (sysctl_ipfrag_size > 7 && frag->len > sysctl_ipfrag_size)
+					goto slow_path_clean;
+				if (frag->len == mtu)
+					goto slow_path_clean;
+			}
+
 			/* Partially cloned skb? */
 			if (skb_shared(frag))
 				goto slow_path_clean;
@@ -727,12 +740,32 @@ slow_path:
 	/*
 	 *	Keep copying data until we run out.
 	 */
+	flen = mtu;
+	if (left > mtu && sysctl_ipfrag_size && sysctl_ipfrag_size < mtu) {
+		if(sysctl_ipfrag_size > 7) {
+			flen = sysctl_ipfrag_size;
+		} else {
+			/* common case flen approx mtu/2 */
+			unsigned int m_frags = left/mtu;
+
+			m_frags += (left % mtu > 0);
+			flen = left/m_frags;
+			flen &= ~7;
+			/* sanity check */
+			if (flen && flen < mtu)
+				while ((flen * m_frags) < left)
+					flen += (left % flen);
+		}
+		/* sanity check */
+		if (flen > mtu || flen < 8)
+			flen = mtu;
+	}
 
 	while (left > 0) {
 		len = left;
-		/* IF: it doesn't fit, use 'mtu' - the data space left */
-		if (len > mtu)
-			len = mtu;
+		/* IF: it doesn't fit, use 'flen (<=mtu)' - the data space left */
+		if (len > flen)
+			len = flen;
 		/* IF: we are not sending up to and including the packet end
 		   then align the next start on an eight byte boundary */
 		if (len < left)	{
